SystemOrganization addCategory: #omnibasePatch!

!Float methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 5/15/2009 14:07'!
asBtreeKeyOfSize: keySize

	^ self asIEEE32BitWord asBtreeKeyOfSize: keySize! !

!ODBObjectIndexFile methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 4/2/2009 16:36'!
swappedObjectSpaces
	"Answer <true> if object space A and B were swapped due to single container reorganization."

	| buf |
	buf := ByteArray new: 1.
	(stream atPosition: self headerLength + 15 getBytesFor: buf) = 1 ifTrue: [^(buf at: 1) = 1].
	OmniBase signalError: 'File read error'! !

!ODBObjectIndexFile methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 4/2/2009 16:36'!
swappedObjectSpaces: aBoolean 
	"Set to <true> if object space A and B were swapped due to single container reorganization."

	| buf |
	buf := ByteArray new: 1.
	buf at: 1 put: (aBoolean ifTrue: [1] ifFalse: [0]).
	stream
		atPosition: self headerLength + 15 putBytesFrom: buf;
		flush! !

!ODBTransaction methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 11/25/2009 17:26'!
abort
	"Abort transaction. After aborting it, it can not be used anymore."

	transactionManager isNil ifFalse: [transactionManager critical: [self basicAbort]].

	self triggerEvent: #finishedTransaction: with: self.
! !

!ODBTransaction methodsFor: '*omnibasePatch' stamp: 'seb 9/4/2011 13:13'!
basicAbort
	"Private - Abort transaction. After aborting it, it can not be used anymore."

	| changes |
	dbConnection isNil 
		ifFalse: 
			[locks isNil 
				ifFalse: 
					[(changes := transactionFile recovery) isNil 
						ifFalse: 
							[changes
								transaction: self;
								rollback.
							transactionFile recovery: nil].
					locks do: 
							[:each | 
							each
								transaction: self;
								unlock]].
			transactionManager transactionAborted: self.
			client transactionRemove: self].
	objects := nil.
	client := nil.
	dbConnection := nil.
	transactionManager := nil.

	objectIndex := nil.
	cacheMutex isNil 
		ifFalse: 
			[cacheMutex terminateProcess.
			cacheMutex := nil]! !

!ODBTransaction methodsFor: '*omnibasePatch' stamp: 'sebastiansastre 9/30/2011 10:24'!
getObjectIDOrNil: anObject 
	"Private - Answer <objectID> coresponding to anObject 
        or <nil> if not allready stored."

	| transactionObject oid |
	anObject odbObjectID isNil ifFalse: [^anObject odbObjectID].

	self isAborted ifTrue:[^nil].

	cacheMutex critical: [transactionObject := objectIndex at: anObject].
	^transactionObject isNil 
		ifFalse: [(oid := transactionObject objectID) index == 0 ifFalse: [oid]]! !

!ODBTransaction methodsFor: '*omnibasePatch' stamp: 'seb 9/3/2011 16:32'!
isAborted

	^ objectIndex isNil and:[
	objects isNil and:[
	client isNil and:[
	dbConnection isNil and:[
	transactionManager isNil and:[
	cacheMutex isNil]]]]]! !

!ODBGarbageCollector methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 21:08'!
basicCompactContainerNamed: aString 
	| container |
	
	[container := objectManager containerNamed: aString
				ifAbsent: [OmniBase signalError: 'Container named ' , aString , ' does not exist'].
	(self checkDiskSpace: (Array with: container)) ifFalse: [^false].
	container
		createNewObjectSpace;
		swapObjectSpaces.
	self compactObjectSpaceForContainer: container reachableObjectsSet: nil] 
			ensure: 
				[self close.
				objectManager reopen.
				objectManager environment globalUnlock]! !

!ODBGarbageCollector methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 21:11'!
basicRun
	
	[self progressNotification: 'Starting garbage collection' value: 0.
	self lockObjectManager.	"doesn't realy lock it, see method #lockObjectManager"
	containers := objectManager containers.
	(self checkDiskSpace: containers) ifFalse: [^false].
	self
		createNewObjectSpaces;
		selectNewObjectSpace;
		swapObjectSpaces;
		walkObjects;
		copyObjects;
		complete] 
			ensure: 
				[self close.
				objectManager reopen.
				objectManager environment globalUnlock]! !

!ODBGarbageCollector methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 21:09'!
checkDiskSpace: containersColl 
	"Private - Check for avaiable disk space and set total number of objects in wait dialog."

	| maxContainerSize n gcSpace |
	maxContainerSize := 0.
	gcSpace := 0.
	containersColl do: 
			[:container | 
			n := container numberOfObjects.
			gcSpace := n // 8 + gcSpace.
			maxContainerSize := container bytesAllocated max: maxContainerSize].
	maxContainerSize + gcSpace < objectManager environment freeDiskSpace ifTrue: [^true].
	^progressDialog isNil 
		ifTrue: [OmniBase logWarning: '[OmniBase GC] Warning disk space low']
		ifFalse: [OmniBase confirmationMessage: 'Disk space low.
Are you sure you want to continue ?']! !

!ODBGarbageCollector methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 22:55'!
compactObjectSpaceForContainer: container reachableObjectsSet: set 
	"Private - Copy compact container objects space and remove objects which are not included in the set. If set is nil, then just compact the object space."

	| dbObject index count |
	count := 0.
	self progressNotification: 'Compacting container ' , container name value: 0.

	container objectHoldersDo: 
			[:holder | 
			count := count + 1.
			self progressNotification: nil value: count / container numberOfObjects.
			index := holder objectID index.
			dbObject := holder getObject.
			(set isNil or: [set includes: index]) 
				ifTrue: [dbObject garbageCollectOn: self]
				ifFalse: 
					[dbObject remove.
					container removeHolderAt: index]].

	container flushBytes! !

!ODBGarbageCollector methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 23:37'!
copyObjects
	"Private - Copy objects from oldObject space to new object space."

	| set objectSpace |
	gcFile status = 5 ifFalse: [^self].
	objectSpace := objectManager defaultObjectSpace.
	containers do: 
			[:container | 
			[set := ODBIntegerSet openOn: (self oidSetFilenameFor: container)] on: Error do: [set := nil].
			set isNil 
				ifFalse: 
					[self compactObjectSpaceForContainer: container reachableObjectsSet: set.
					set remove.
					container removeObjectSpace: 1 - objectSpace]].
	gcFile status: 6! !

!ODBGarbageCollector methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 21:19'!
openOn: anOmniBase 
	"Open garbage collector on anOmnibase. If garbage collector file exists
	then previous garbage collection was interrupted.
	Answer <true> if previous GC was completed, <false> if it wasn't."

	objectManager := anOmniBase objectManager.
	(ODBFileStream exists: self gcFileName) 
		ifTrue: 
			[(gcFile := ODBGarbageCollectorFile openOn: self) isNil 
				ifFalse: 
					[gcFile status < 3 
						ifTrue: 
							[gcFile remove.
							gcFile := nil]]].
	^gcFile isNil! !

!ODBWin32FileStream methodsFor: '*omnibasePatch' stamp: 'sas 2/26/2009 20:49'!
basicGetBytesFor: aByteCollection len: len 
	"Private - Read len bytes from stream to aByteCollection. 
	Answer number of bytes actualy read."

	| startTime currentTime errorCode |
	(self 
		readFile: fileHandle
		lpBuffer: aByteCollection
		nNumberOfBytesToRead: len
		lpNumberOfBytesRead: externalLong
		lpOverlapped: nil) = 0
			ifTrue: 
				[currentTime := startTime := Time millisecondClockValue.
				
				[errorCode := self getLastError.
				errorCode = 33 or: [errorCode = 5	"access denied"]	"lock violation"] 
						whileTrue: 
							[currentTime - startTime < self maxMillisecondsOfTolerance 
								ifFalse: 
									[ODBStreamLockedNotification signal 
										ifTrue: [startTime := currentTime]
										ifFalse: [self fileIOError: errorCode]].
							(self 
								readFile: fileHandle
								lpBuffer: aByteCollection
								nNumberOfBytesToRead: len
								lpNumberOfBytesRead: externalLong
								lpOverlapped: nil) = 0 ifFalse: [^externalLong odbLongAt: 1].
							currentTime := Time millisecondClockValue.
							currentTime < startTime ifTrue: [currentTime := currentTime + 86400000]].
				self fileIOError: errorCode].
	^externalLong odbLongAt: 1! !

!ODBWin32FileStream methodsFor: '*omnibasePatch' stamp: 'sas 2/26/2009 21:29'!
maxMillisecondsOfTolerance 
	
	^ 500  "let's see what happens with this"
	
	"^ 12  OmniBase default"! !

!ODBContainer methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 21:09'!
bytesAllocated
	| size1 size2 |
	size1 := 0.
	size2 := 0.
	dataFileA isNil ifFalse: [size1 := dataFileA size].
	dataFileB isNil ifFalse: [size2 := dataFileB size].
	^size1 + size2! !

!ODBContainer methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 19:48'!
convertToLocalObjectSpaceNumber: objectSpaceNumber 
	^indexFile swappedObjectSpaces ifTrue: [1 - objectSpaceNumber] ifFalse: [objectSpaceNumber]! !

!ODBContainer methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 19:59'!
localDefaultObjectSpace
	^self convertToLocalObjectSpaceNumber: objectManager defaultObjectSpace! !

!ODBContainer methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/30/2010 10:41'!
lockDataFile

	"Dec, 2010 sas: this code was relying in evaluating a block that was happening only in win32
	but not in *nix. So now I'm making it wait for the lock and then execute the code normally
	because it seems the intention was the same."
	| currentSpace defaultSpace |

	activeDataFile waitForAddingLock.
	currentSpace := activeDataFile == dataFileA ifTrue: [0] ifFalse: [1].
	defaultSpace := objectManager defaultObjectSpace.
	currentSpace == defaultSpace 
		ifFalse: 
			[defaultSpace == 0 
				ifTrue: 
					[dataFileA isNil ifTrue: [dataFileA := ODBObjectStorage openOn: self dataFileNameA].
					activeDataFile := dataFileA]
				ifFalse: 
					[dataFileB isNil ifTrue: [dataFileB := ODBObjectStorage openOn: self dataFileNameB].
					activeDataFile := dataFileB].
			^self lockDataFile]! !

!ODBContainer methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 19:59'!
openOn: aString 
	path := aString.
	
	[indexFile := ODBObjectIndexFile openOn: self indexFileName.
	[dataFileA := ODBObjectStorage openOn: self dataFileNameA] on: Error do: [:ex | ].
	[dataFileB := ODBObjectStorage openOn: self dataFileNameB] on: Error do: [:ex | ].
	self localDefaultObjectSpace = 0 
		ifTrue: [(activeDataFile := dataFileA) isNil ifTrue: [activeDataFile := dataFileB]]
		ifFalse: [(activeDataFile := dataFileB) isNil ifTrue: [activeDataFile := dataFileA]].
	activeDataFile isNil 
		ifTrue: [OmniBase signalError: 'Object storage file ' , self dataFileNameA , ' can not be opened']] 
			ifCurtailed: [self close]! !

!ODBContainer methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/30/2010 00:37'!
removeObjectFile: anInteger 
	| file |
	dbFilesMutex critical: 
			[file := dbFiles at: anInteger ifAbsent: [^self].
			dbFiles removeKey: anInteger.
			file ifNotNil:[
				file close].
			ODBFileStream remove: (self fileNameFromIndex: anInteger)]! !

!ODBContainer methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 23:16'!
removeObjectSpace: anInteger 

	(self convertToLocalObjectSpaceNumber: anInteger) = 0 
		ifTrue: 
			[dataFileA isNil 
				ifFalse: 
					[activeDataFile == dataFileA ifTrue: [activeDataFile := dataFileB].
					dataFileA
						unlockAdding;
						removeAll;
						remove.
					dataFileA := nil]]
		ifFalse: 
			[dataFileB isNil 
				ifFalse: 
					[activeDataFile == dataFileB ifTrue: [activeDataFile := dataFileA].
					dataFileB
						unlockAdding;
						removeAll;
						remove.
					dataFileB := nil]]! !

!ODBContainer methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/30/2010 10:24'!
setNewObjectSpace: newSpace 
	| currentSpace newLocalSpace |

	newLocalSpace := self convertToLocalObjectSpaceNumber: newSpace.
	currentSpace := activeDataFile == dataFileA ifTrue: [0] ifFalse: [1].
	currentSpace = newLocalSpace ifTrue: [^self].
	newLocalSpace = 0 
		ifTrue: 
			[dataFileA isNil ifTrue: [dataFileA := ODBObjectStorage openOn: self dataFileNameA].
			activeDataFile := dataFileA]
		ifFalse: 
			[dataFileB isNil ifTrue: [dataFileB := ODBObjectStorage openOn: self dataFileNameB].
			activeDataFile := dataFileB]! !

!OmniBase class methodsFor: '*omnibasePatch' stamp: 'sas 2/27/2009 09:18'!
reset
	"Use wisely. Makes a complete reset of all related to this persistance clients."
	self closeAll.
	processToTransactionDict := IdentityDictionary new.
	processToTransactionMutex := Semaphore forMutualExclusion.
	currentTransaction := nil.
	! !

!OmniBase methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 4/6/2009 10:56'!
containerNamed: aName ifAbsent: aBlock
	"Answers the container the receiver locates 
	at aName or executes aBlock if not found."

	^ objectManager containerNamed: aName ifAbsent: aBlock ! !

!OmniBase methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 3/7/2009 18:34'!
printOn: aStream

	super printOn: aStream.
	
	aStream nextPutAll: '(',self hash asString,')'! !

!ODBReference class methodsFor: '*omnibasePatch' stamp: 'sebastiansastre 1/4/2014 10:44'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ Smalltalk tools inspector! !

!ODBReference methodsFor: '*omnibasePatch' stamp: 'seb 9/3/2011 17:28'!
defaultLabelForInspector
	"Answer the default label to be used for an Inspector window on the receiver."

	^ self class name! !

!ODBReference methodsFor: '*omnibasePatch' stamp: 'seb 9/3/2011 17:52'!
halt
	"This is the typical message to use for inserting breakpoints during 
	debugging. It behaves like halt:, but does not call on halt: in order to 
	avoid putting this message on the stack. Halt is especially useful when 
	the breakpoint message is an arbitrary one."

	Halt signal! !

!ODBReference methodsFor: '*omnibasePatch' stamp: 'sebastiansastre 1/4/2014 10:42'!
inspect
	"Create and schedule an Inspector in which the user can examine the receiver's variables."
	Smalltalk tools inspector inspector: self! !

!ODBReference methodsFor: '*omnibasePatch' stamp: 'seb 9/3/2011 17:24'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ Smalltalk tools inspector! !

!Date methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 3/8/2009 18:41'!
asBtreeKeyOfSize: keySize

	keySize < 3 ifTrue: [OmniBase signalError: 'Key size has to be at least 3 bytes to accomodate a date'].
	^self julianDayNumber asBtreeKeyOfSize: keySize! !

!Date methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 3/8/2009 18:40'!
odbBasicSerialize: serializer

	serializer stream putByte: 32; putInteger: self julianDayNumber - 2415386! !

!ODBObjectID methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/23/2010 10:45'!
= other

    ^ self species = other species and: [
	other index == index and: [
	other containerID == containerID]]! !

!ODBLocalTransaction methodsFor: '*omnibasePatch' stamp: 'sebastiansastre 9/30/2011 10:17'!
checkpoint
	"Checkpoint transaction i.e. write all persistent objects that are marked as dirty or new into database.
	Only changes are written into the database. This is an atomic operation which will either complete successfully
	or fail completely with no changes whatsoever written into the database. After writing changes all locks on already
	existing object will still be set. Objects that were new, had no locks set and are thus not locked. Transaction can
	be checkpointed more than once. As seen below the whole process of writing changes into the database (files)
	is executed using a semaphore therefore only one transaction per database session can write its changes at a time."

	self isInconsistent ifTrue: [OmniBase signalError: 'Transaction is inconsistent'].
	transactionManager ifNil:[ODBTransactionCanNotCommit signal].
	
	transactionManager critical: 
			[[self prepare] ifCurtailed: 
					[self becomeInconsistent.
					transactionManager transactionCommitEnd: self].
			[changesPackage commit] ifCurtailed: 
					[self
						becomeInconsistent;
						basicAbort]].
	self committed! !

!ODBLocalTransaction methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 11/25/2009 16:14'!
commit

	"Transcript cr; show: 'committing t: ', self hash asString,' p: ',Processor activeProcess hash asString."
	
	self
		checkpoint;
		abort! !

!ODBLocalTransaction methodsFor: '*omnibasePatch' stamp: 'sas 2/27/2009 09:20'!
locks
	^ locks ! !

!ODBLocalTransaction methodsFor: '*omnibasePatch' stamp: 'sebastiansastre 9/30/2011 12:17'!
makePersistent: anObject in: aContainerOrContainerName 
	"Store anObject into specified container if it is not stored yet, else do nothing.
	Answer anObject."

	| container |
	anObject odbObjectID isNil ifFalse: [^anObject].
	self getTransactionObject: anObject
		ifAbsent: 
			[(aContainerOrContainerName isKindOf: String)
				ifTrue: 
					[container := dbConnection objectManager containerNamed: aContainerOrContainerName
								ifAbsent: [OmniBase signalError: 'Container does not exist']]
				ifFalse: [container := aContainerOrContainerName].
			self storeNewObject: anObject inContainer: container].
	^anObject! !

!ODBLocalTransaction methodsFor: '*omnibasePatch' stamp: 'sebastiansastre 9/20/2011 18:08'!
markDirty: anObject 
	"Mark anObject as dirty so that it will be written to the 
	database when message #commit is sent.
	Note: if an object is not loaded yet i.e. the message #markDirty is sent to an unresolved ODBReference the object will not be marked as dirty.
	Therefore send #markDirty after you've loaded and changed the object, not before, else use #odbResolve before sending #markDirty."

	| objectToStore transactionObject |
	cacheMutex isNil 
		ifTrue: 
			["transaction is already aborted"
			ODBTransactionAlreadyAborted signal: 'Transaction is already aborted.'].
	(objectToStore := anObject) odbObjectID isNil 
		ifFalse: 
			[anObject isODBReference ifFalse: [^self].
			anObject odbNotLoaded ifTrue: [^self].
			anObject odbTransaction == self ifFalse: [objectToStore := anObject odbResolve]].
	transactionObject := self getTransactionObject: objectToStore
				ifAbsent:[
					ODBNotPersistent signal: 'Object is not persistent or is a part of composite persistent object'].
	transactionObject objectID index = 0 ifTrue: [^true].
	transactionObject lock ifFalse: [^self signalLockingConflictAt: anObject].
	transactionObject objectChanged! !

!ODBLocalTransaction methodsFor: '*omnibasePatch' stamp: 'sas 2/26/2009 21:34'!
printOn: aStream

	super printOn: aStream.
	
	aStream nextPutAll: '(',self hash asString,')'! !

!ODBFile methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 21:33'!
size 

	^ stream size! !

!ODBFile methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/30/2010 11:14'!
waitForLockAt: pos length: length whileWaitingDo: aBlock
		"Wait for lock at given position. 
		Evaluate aBlock in each iteration."

	"Dec, 2010. sas: evaluating code in each iteration while waiting sounds weird but whatever.
	I think this was designed on the wrong assumption that the OS always will provide a false 
	in the first try but what actually happens is that in win32 you always will 
	get at least one false but in *nixes you don't do see that happening (which is totally reasonable).
	BEWARE of stupid code relying on this stupid assumption (I've already patched #lockDataFile because of this)."
	
	| startTime currentTime |
	(stream lockAt: pos length: length) ifFalse: [
		startTime := Time totalSeconds.
		[stream lockAt: pos length: length] whileFalse: [
			aBlock value.
			currentTime := Time totalSeconds.
			currentTime - startTime < self timeOutLength
				ifFalse: [
					ODBLockNotification signal
						ifTrue: [startTime := currentTime]
						ifFalse: [^ODBCannotLock signal]]]]! !

!ODBFileStream class methodsFor: '*omnibasePatch' stamp: 'sebastiansastre 4/17/2011 00:27'!
createOn: pathName createMode: createMode accessMode: accessMode shareMode: shareMode cacheMode: cacheMode
		"Create new instance of file stream on pathName with given flags."

	| fileStreamClass |
	fileStreamClass := Smalltalk os platformName
			caseOf: 
				{
				['unix'] -> [ODBUnixFileStream].
				['Mac OS'] -> [ODBMacFileStream].
				['Win32'] -> [ODBWin32FileStream].
				}
			otherwise: 
				[self notify: 'File sharing & locking not implemented for this platform'.
				^ self 
					createWithoutSharingAndLockingOn: pathName 
					accessMode: accessMode].
			
	^ fileStreamClass
		createOn: pathName 
		createMode: createMode 
		accessMode: accessMode 
		shareMode: shareMode 
		cacheMode: cacheMode! !

ODBFileStream subclass: #ODBMacFileStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'omnibasePatch'!

!ODBMacFileStream class methodsFor: 'private' stamp: 'sebastian.sastre 6/17/2010 16:51'!
createFileStreamForPath: pathString createMode: createMode
	^ FileStream concreteStream
			perform: (self streamCreationSelectorForMode: createMode)
			with: pathString! !

!ODBMacFileStream class methodsFor: 'create/open' stamp: 'sebastian.sastre 9/25/2010 12:57'!
createOn: pathString createMode: createMode accessMode: accessMode shareMode: shareMode cacheMode: cacheMode
	| stream needsWrite |

	stream := self createFileStreamForPath: pathString createMode: createMode.
	
	^	[self ensureAccessMode: accessMode forFileStream: stream.
		self setShareMode: shareMode forFileStream: stream.
	
		self new openOn: pathString fileHandle: stream]
			ifCurtailed: [stream close. nil]! !

!ODBMacFileStream class methodsFor: 'private' stamp: 'sebastian.sastre 9/25/2010 12:57'!
ensureAccessMode: accessMode forFileStream: aFileStream
	"Check the special byte is appropriately set to allow the desired access mode.  Signal a locking conflict if there is a problem.
	See the class comment for more information on what's going on here."
	
	| needsWriteAccess |
	needsWriteAccess := (accessMode = self accessModeReadOnly) not.
	(OSProcess accessor
		isLockableFile: aFileStream
		from: self shareModeFlagLocation
		to: self shareModeFlagLocation
		exclusive: needsWriteAccess)
			ifFalse: [ODBLockingConflict signal].! !

!ODBMacFileStream class methodsFor: 'locking' stamp: 'sebastian.sastre 6/17/2010 16:51'!
releaseAllLocksFor: pathName fileHandle: fileHandle
	OSProcess accessor unlockAllForFile: fileHandle! !

!ODBMacFileStream class methodsFor: 'private' stamp: 'sebastian.sastre 9/25/2010 12:57'!
setShareMode: shareMode forFileStream: aFileStream
	"Set the special byte appropriately for the desired share mode.  Signal a locking conflict if there is a problem.
	See the class comment for more information on what's going on here."
	
	| flagState |
	flagState := shareMode caseOf: {
			[self shareModeShareAll] -> [nil].
			[self shareModeShareRead] -> [false].
			[self shareModeShareNone] -> [true].
			[self shareModeShareWrite] -> [self halt] "don't know how to implement this on unix and it is currently unused"}.

	flagState ifNotNil:
		[(OSProcess accessor
			lockFile: aFileStream
			from: self shareModeFlagLocation
			to: self shareModeFlagLocation
			exclusive: flagState)
				ifNil: [ODBLockingConflict signal]].! !

!ODBMacFileStream class methodsFor: 'private' stamp: 'sebastian.sastre 6/17/2010 16:51'!
shareModeFlagLocation
	"This is the byte location at which the share mode lock is made (see the class comment for details).  This value should probably be less than ODBObjectHolder>>lockStartPosition to avoid conflicts, though it appears that the index file is currently always opened with #shareModeShareAll, so we are actually probably fine in practice."
	
	^ 999999999! !

!ODBMacFileStream class methodsFor: 'private' stamp: 'sebastian.sastre 6/17/2010 16:51'!
streamCreationSelectorForMode: createMode
	^ createMode caseOf:  {
		[self createModeCreateAlways] -> [#forceNewFileNamed:].
		[self createModeCreateNew] -> [#newFileNamed:].
		[self createModeOpenAlways] -> [#fileNamed:].
		[self createModeOpenExisting] -> [#oldFileNamed:]}
! !

!ODBMacFileStream methodsFor: 'public' stamp: 'sebastian.sastre 6/17/2010 16:51'!
lockAt: pos length: length
	^ (OSProcess accessor lockFile: fileHandle from: pos to: pos + length -1) notNil! !

!ODBMacFileStream methodsFor: 'public' stamp: 'sebastian.sastre 6/17/2010 16:54'!
size
	"the unix Squeak VM gives the wrong answer for #size"
	| file |
	fileHandle flush.
	file := fileHandle readOnlyCopy.
	^ [file size] ensure: [file close]
	

! !

!ODBMacFileStream methodsFor: 'public' stamp: 'sebastian.sastre 6/17/2010 16:51'!
unlockAt: pos length: length
	^ (OSProcess accessor unlockFile: fileHandle from: pos to: pos + length -1) notNil! !

!DateAndTime methodsFor: '*omnibasePatch' stamp: 'sebastiansastre 12/19/2011 16:25'!
asBtreeKeyOfSize: keySize 

	keySize < 8 ifTrue: [
		OmniBase signalError: 'Key size has to be at least 8 bytes to accomodate a DateAndTime'].

	^self asSeconds asBtreeKeyOfSize: keySize! !

!ODBLocalClient methodsFor: '*omnibasePatch' stamp: 'sas 2/27/2009 10:49'!
makeRootObject
	"Private - Initialize root object."

	[:txn|
		txn
			root: ODBPersistentDictionary new;
			commit] value: self newTransaction! !

Object subclass: #BlockContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'omnibasePatch'!

!Object class methodsFor: '*omnibasePatch' stamp: 'sebastian.sastre 12/29/2010 18:00'!
odbLoadVariables: deserializer classInfo: classInfo

	| stream order iSize object basicSize varArray |
	stream := deserializer stream.
	classInfo isVariable
		ifTrue: [ object := self basicNew: (basicSize := stream getPositiveInteger) ]
		ifFalse: [
			basicSize := 0.
			object := self basicNew.
		].
	deserializer register: object.
	classInfo isBytes ifTrue: [
		classInfo realClass isWords 
			ifTrue: [1 to: basicSize do: [:i | object basicAt: i put: stream getInteger]]
			ifFalse: [stream getBytesFor: object len: basicSize].
		^object
	].
	varArray := Array new: (iSize := classInfo realInstSize) + 1.
	order := classInfo variableOrder.
	1 to: classInfo instSize do: [:i | varArray at: (order at: i) put: deserializer odbNextObject ].
	1 to: iSize do: [:i | object instVarAt: i put: (varArray at: i) ].
	iSize + 1 to: iSize + basicSize do: [:i | object instVarAt: i put: deserializer odbNextObject ].
	^object odbDeserialized: deserializer! !

!Object methodsFor: '*omnibasepatch' stamp: 'sebastiansastre 12/19/2011 16:50'!
makePersistent
	"Makes the object persistent in the current process's transaction.
	Note: because is not specifiying where, this instance is going to 
	be persisted in the default container."

	"Note: using a dynamic variable could allow to still use the #makePersistent message
	while deciding somewhere else in which container this should be stored.
	It would look like:
	ARCurrentContainer value isNil
		ifTrue:[OmniBase currentTransaction makePersistent: self]
		ifFalse:[OmniBase currentTransaction makePersistent: self in: ARCurrentContainer value]"
	
	OmniBase currentTransaction makePersistent: self
		! !

!Object methodsFor: '*omnibasepatch' stamp: 'sebastiansastre 9/30/2011 17:19'!
odbGetOID
	"Answer persistent object's OID."

	| t oid |
	(oid := self odbObjectID) isNil ifFalse: [^oid].
	(t := OmniBase currentTransactionOrNil) isNil 
		ifFalse: [(oid := t getObjectIDOrNil: self) isNil ifFalse: [^oid]].

	"sas: declared that bug when trying #odbGetTransaction on some odb reference that's a subinstance
	of this persistent and fails due to strangeness in instVars is an irrelevant, inconsequent mistery that
	only complicates one case that isn't that important after all. So I'm commenting the following lines:
	
	t isAborted ifTrue:[^ nil]."
	"(t := self odbGetTransaction) isNil ifFalse: [^t getObjectIDOrNil: self]."
	^nil! !

!Object methodsFor: '*omnibasepatch' stamp: 'sebastiansastre 9/30/2011 17:06'!
odbGetTransaction
	"Answer the transaction in which the object was loaded."

	"immediate objects can not be made persistent"

	| allReferences |
	self isImmediate ifTrue: [^nil].

	"first check all inst vars if any references a proxy from which we could get a reference to transaction"
	self class isBits
		ifFalse: 
			[1 to: self class instSize + self basicSize
				do: [:i | (self instVarAt: i) isODBReference ifTrue: [^(self instVarAt: i) odbTransaction]]].

	"check references to self and find the transaction in which object was loaded"	
	"allReferences := self pointersTo.
	allReferences do: [:each | each isODBReference ifTrue: [^each odbTransaction]]."
	^nil! !

Object subclass: #TranscriptStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'omnibasePatch'!

Object subclass: #WAEmailMessage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'omnibasePatch'!
